@@ -8,10 +8,12 @@ from moto.iam import models as iam_models
 from moto.sqs import models as sqs_models
 from moto.core import BaseModel
 from moto.server import main as moto_main
+from moto.kinesis import models as kinesis_models
 from moto.dynamodb import models as dynamodb_models
 from moto.dynamodb2 import models as dynamodb2_models
 from moto.awslambda import models as lambda_models
 from moto.apigateway import models as apigw_models
+from moto.cloudwatch import models as cw_models
 from moto.cloudformation import parsing, responses
 from boto.cloudformation.stack import Output
 from moto.cloudformation.exceptions import ValidationError, UnformattedGetAttTemplateException
@@ -44,7 +46,8 @@ MODEL_MAP = {
     'AWS::ApiGateway::Method': apigw_models.Method,
     'AWS::ApiGateway::Resource': apigw_models.Resource,
     'AWS::ApiGateway::RestApi': apigw_models.RestAPI,
-    'AWS::StepFunctions::StateMachine': sfn_models.StateMachine
+    'AWS::StepFunctions::StateMachine': sfn_models.StateMachine,
+    'AWS::CloudFormation::Stack': service_models.CloudFormationStack
 }
 
 
@@ -128,6 +131,8 @@ def update_physical_resource_id(resource):
         elif isinstance(resource, service_models.StepFunctionsActivity):
             act_arn = aws_stack.stepfunctions_activity_arn(resource.params.get('Name'))
             resource.physical_resource_id = act_arn
+        elif isinstance(resource, kinesis_models.Stream):
+            resource.physical_resource_id = resource.stream_name
         else:
             LOG.warning('Unable to determine physical_resource_id for resource %s' % type(resource))
 
@@ -399,6 +404,16 @@ def apply_patches():
     SQS_Queue_get_cfn_attribute_orig = sqs_models.Queue.get_cfn_attribute
     sqs_models.Queue.get_cfn_attribute = SQS_Queue_get_cfn_attribute
 
+    # Patch S3 Bucket get_cfn_attribute(..) method in moto
+
+    def S3_Bucket_get_cfn_attribute(self, attribute_name):
+        if attribute_name in ['Arn']:
+            return aws_stack.s3_bucket_arn(self.name)
+        return S3_Bucket_get_cfn_attribute_orig(self, attribute_name)
+
+    S3_Bucket_get_cfn_attribute_orig = s3_models.FakeBucket.get_cfn_attribute
+    s3_models.FakeBucket.get_cfn_attribute = S3_Bucket_get_cfn_attribute
+
     # Patch SQS physical_resource_id(..) method in moto
 
     @property
@@ -412,6 +427,19 @@ def apply_patches():
     SQS_Queue_physical_resource_id_orig = sqs_models.Queue.physical_resource_id
     sqs_models.Queue.physical_resource_id = SQS_Queue_physical_resource_id
 
+    # Patch LogGroup get_cfn_attribute(..) method in moto
+
+    def LogGroup_get_cfn_attribute(self, attribute_name):
+        try:
+            return LogGroup_get_cfn_attribute_orig(self, attribute_name)
+        except Exception:
+            if attribute_name == 'Arn':
+                return aws_stack.log_group_arn(self.name)
+            raise
+
+    LogGroup_get_cfn_attribute_orig = getattr(cw_models.LogGroup, 'get_cfn_attribute', None)
+    cw_models.LogGroup.get_cfn_attribute = LogGroup_get_cfn_attribute
+
     # Patch Lambda get_cfn_attribute(..) method in moto
 
     def Lambda_Function_get_cfn_attribute(self, attribute_name):
@@ -613,13 +641,17 @@ def apply_patches():
 
     def cf_describe_stack_events(self):
         stack_name = self._get_param('StackName')
-        stack = self.cloudformation_backend.get_stack(stack_name)
+        backend = self.cloudformation_backend
+        stack = backend.get_stack(stack_name)
+        if not stack:
+            # Also return stack events for deleted stacks, specified by stack name
+            stack = ([stk for id, stk in backend.deleted_stacks.items() if stk.name == stack_name] or [0])[0]
         if not stack:
             raise ValidationError(stack_name,
                 message='Unable to find stack "%s" in region %s' % (stack_name, aws_stack.get_region()))
-        return cf_describe_stack_events_orig(self)
+        template = self.response_template(responses.DESCRIBE_STACK_EVENTS_RESPONSE)
+        return template.render(stack=stack)
 
-    cf_describe_stack_events_orig = responses.CloudFormationResponse.describe_stack_events
     responses.CloudFormationResponse.describe_stack_events = cf_describe_stack_events
 
 
