@@ -303,7 +303,7 @@ def append_last_modified_headers(response, content=None):
     time_format = '%a, %d %b %Y %H:%M:%S GMT'  # TimeFormat
     try:
         if content:
-            last_modified_str = re.findall(r'<LastModified>(.*)</LastModified>', content)
+            last_modified_str = re.findall(r'<LastModified>([^<]*)</LastModified>', content)
             if last_modified_str:
                 last_modified_str = last_modified_str[0]
                 last_modified_time_format = dateutil.parser.parse(last_modified_str).strftime(time_format)
@@ -354,6 +354,43 @@ def fix_location_constraint(response):
         remove_xml_preamble(response)
 
 
+def fix_range_content_type(bucket_name, path, headers, response):
+    # Fix content type for Range requests - https://github.com/localstack/localstack/issues/1259
+    if 'Range' not in headers:
+        return
+
+    s3_client = aws_stack.connect_to_service('s3')
+    path = urlparse.unquote(path)
+    key_name = get_key_name(path, headers)
+    result = s3_client.head_object(Bucket=bucket_name, Key=key_name)
+    content_type = result['ContentType']
+    if response.headers.get('Content-Type') == 'text/html; charset=utf-8':
+        response.headers['Content-Type'] = content_type
+
+
+def fix_delete_objects_response(bucket_name, method, parsed_path, data, headers, response):
+    # Deleting non-existing keys should not result in errors.
+    # Fixes https://github.com/localstack/localstack/issues/1893
+    if not (method == 'POST' and parsed_path.query == 'delete' and '<Delete' in to_str(data or '')):
+        return
+    content = to_str(response._content)
+    if '<Error>' not in content:
+        return
+    result = xmltodict.parse(content).get('DeleteResult')
+    errors = result.get('Error')
+    errors = errors if isinstance(errors, list) else [errors]
+    deleted = result.get('Deleted')
+    if not isinstance(result.get('Deleted'), list):
+        deleted = result['Deleted'] = [deleted] if deleted else []
+    for entry in list(errors):
+        if set(entry.keys()) == set(['Key']):
+            errors.remove(entry)
+            deleted.append(entry)
+    if not errors:
+        result.pop('Error')
+    response._content = xmltodict.unparse({'DeleteResult': result})
+
+
 def remove_xml_preamble(response):
     """ Removes <?xml ... ?> from a response content """
     response._content = re.sub(r'^<\?[^\?]+\?>', '', response._content)
@@ -556,16 +593,28 @@ def normalize_bucket_name(bucket_name):
     return bucket_name
 
 
+def get_key_name(path, headers):
+    parsed = urlparse.urlparse(path)
+    path_parts = parsed.path.lstrip('/').split('/', 1)
+
+    if uses_path_addressing(headers):
+        return path_parts[1]
+    return path_parts[0]
+
+
+def uses_path_addressing(headers):
+    host = headers['host']
+    return host.startswith(HOSTNAME) or host.startswith(HOSTNAME_EXTERNAL)
+
+
 def get_bucket_name(path, headers):
     parsed = urlparse.urlparse(path)
 
     # try pick the bucket_name from the path
     bucket_name = parsed.path.split('/')[1]
 
-    host = headers['host']
-
-    # is the hostname not starting a bucket name?
-    if host.startswith(HOSTNAME) or host.startswith(HOSTNAME_EXTERNAL):
+    # is the hostname not starting with a bucket name?
+    if uses_path_addressing(headers):
         return normalize_bucket_name(bucket_name)
 
     # matches the common endpoints like
@@ -585,6 +634,7 @@ def get_bucket_name(path, headers):
 
     # if any of the above patterns match, the first captured group
     # will be returned as the bucket name
+    host = headers['host']
     for pattern in [common_pattern, dualstack_pattern, legacy_patterns]:
         match = pattern.match(host)
         if match:
@@ -816,6 +866,8 @@ class ProxyListenerS3(ProxyListener):
             if response.status_code == 200 and status_code == '201' and key:
                 response.status_code = 201
                 response._content = self.get_201_reponse(key, bucket_name)
+                response.headers['Content-Length'] = str(len(response._content))
+                response.headers['Content-Type'] = 'application/xml; charset=utf-8'
                 return response
 
         parsed = urlparse.urlparse(path)
@@ -884,6 +936,8 @@ class ProxyListenerS3(ProxyListener):
             append_last_modified_headers(response=response)
             append_list_objects_marker(method, path, data, response)
             fix_location_constraint(response)
+            fix_range_content_type(bucket_name, path, headers, response)
+            fix_delete_objects_response(bucket_name, method, parsed, data, headers, response)
 
             # Remove body from PUT response on presigned URL
             # https://github.com/localstack/localstack/issues/1317
