@@ -86,14 +86,43 @@ class ProxyListenerSQS(ProxyListener):
         if method != 'POST':
             return
 
-        if response.status_code >= 400:
-            return response
-
         region_name = extract_region_from_auth_header(headers)
         req_data = urlparse.parse_qs(to_str(data))
         action = req_data.get('Action', [None])[0]
         content_str = content_str_original = to_str(response.content)
 
+        if response.status_code >= 400:
+
+            # Since the following 2 API calls are not implemented in ElasticMQ, we're mocking them
+            # and letting them to return an empty response
+            if action == 'TagQueue':
+                new_response = Response()
+                new_response.status_code = 200
+                new_response._content = ("""
+                    <?xml version="1.0"?>
+                    <TagQueueResponse>
+                        <ResponseMetadata>
+                            <RequestId>{}</RequestId>
+                        </ResponseMetadata>
+                    </TagQueueResponse>
+                """).strip().format(uuid.uuid4())
+                return new_response
+            elif action == 'ListQueueTags':
+                new_response = Response()
+                new_response.status_code = 200
+                new_response._content = ("""
+                    <?xml version="1.0"?>
+                    <ListQueueTagsResponse xmlns="{}">
+                        <ListQueueTagsResult/>
+                        <ResponseMetadata>
+                            <RequestId>{}</RequestId>
+                        </ResponseMetadata>
+                    </ListQueueTagsResponse>
+                """).strip().format(XMLNS_SQS, uuid.uuid4())
+                return new_response
+
+            return response
+
         self._fire_event(req_data, response)
 
         # patch the response and add missing attributes
@@ -122,34 +151,6 @@ class ProxyListenerSQS(ProxyListener):
             new_response.headers['content-length'] = len(new_response._content)
             return new_response
 
-        # Since the following 2 API calls are not implemented in ElasticMQ, we're mocking them
-        # and letting them to return an empty response
-        if action == 'TagQueue':
-            new_response = Response()
-            new_response.status_code = 200
-            new_response._content = ("""
-                <?xml version="1.0"?>
-                <TagQueueResponse>
-                    <ResponseMetadata>
-                        <RequestId>{}</RequestId>
-                    </ResponseMetadata>
-                </TagQueueResponse>
-            """).strip().format(uuid.uuid4())
-            return new_response
-        elif action == 'ListQueueTags':
-            new_response = Response()
-            new_response.status_code = 200
-            new_response._content = ("""
-                <?xml version="1.0"?>
-                <ListQueueTagsResponse xmlns="{}">
-                    <ListQueueTagsResult/>
-                    <ResponseMetadata>
-                        <RequestId>{}</RequestId>
-                    </ResponseMetadata>
-                </ListQueueTagsResponse>
-            """).strip().format(XMLNS_SQS, uuid.uuid4())
-            return new_response
-
     # Format of the message Name attribute is MessageAttribute.<int id>.<field>
     # Format of the Value attributes is MessageAttribute.<int id>.Value.DataType
     # and MessageAttribute.<int id>.Value.<Type>Value
@@ -260,7 +261,7 @@ class ProxyListenerSQS(ProxyListener):
 
     def _send_message(self, path, data, req_data, headers):
         queue_url = self._queue_url(path, req_data, headers)
-        queue_name = queue_url[queue_url.rindex('/') + 1:]
+        queue_name = queue_url.rpartition('/')[2]
         message_body = req_data.get('MessageBody', [None])[0]
         message_attributes = self.format_message_attributes(req_data)
         region_name = extract_region_from_auth_header(headers)
