@@ -13,6 +13,7 @@ import socket
 import hashlib
 import decimal
 import logging
+import tarfile
 import zipfile
 import binascii
 import calendar
@@ -63,6 +64,10 @@ LOG = logging.getLogger(__name__)
 # flag to indicate whether we've received and processed the stop signal
 INFRA_STOPPED = False
 
+# generic cache object
+CACHE = {}
+
+# lock for creating certificate files
 SSL_CERT_LOCK = threading.RLock()
 
 
@@ -558,6 +563,12 @@ def rm_rf(path):
     """
     if not path or not os.path.exists(path):
         return
+    # Running the native command can be an order of magnitude faster in Alpine on Travis-CI
+    if is_alpine():
+        try:
+            return run('rm -rf "%s"' % path)
+        except Exception:
+            pass
     # Make sure all files are writeable and dirs executable to remove
     chmod_r(path, 0o777)
     # check if the file is either a normal file, or, e.g., a fifo
@@ -582,6 +593,10 @@ def download(url, path, verify_ssl=True):
     # enable parallel file downloads during installation!
     s = requests.Session()
     r = s.get(url, stream=True, verify=verify_ssl)
+    # check status code before attempting to read body
+    if r.status_code >= 400:
+        raise Exception('Failed to download %s, response code %s' % (url, r.status_code))
+
     total = 0
     try:
         if not os.path.exists(os.path.dirname(path)):
@@ -597,8 +612,13 @@ def download(url, path, verify_ssl=True):
                     LOG.debug('Empty chunk %s (total %s) from %s' % (chunk, total, url))
             f.flush()
             os.fsync(f)
+        if os.path.getsize(path) == 0:
+            LOG.warning('Zero bytes downloaded from %s, retrying' % url)
+            download(url, path, verify_ssl)
+            return
+        LOG.debug('Done downloading %s, response code %s, total bytes %d' % (url, r.status_code, total))
     finally:
-        LOG.debug('Done downloading %s, response code %s' % (url, r.status_code))
+        LOG.debug('Cleaning up file handles for download of %s' % url)
         r.close()
         s.close()
 
@@ -637,12 +657,15 @@ def is_linux():
 
 def is_alpine():
     try:
-        if not os.path.exists('/etc/issue'):
-            return False
-        out = to_str(subprocess.check_output('cat /etc/issue', shell=True))
-        return 'Alpine' in out
+        if '_is_alpine_' not in CACHE:
+            CACHE['_is_alpine_'] = False
+            if not os.path.exists('/etc/issue'):
+                return False
+            out = to_str(subprocess.check_output('cat /etc/issue', shell=True))
+            CACHE['_is_alpine_'] = 'Alpine' in out
     except subprocess.CalledProcessError:
         return False
+    return CACHE['_is_alpine_']
 
 
 def get_arch():
@@ -774,6 +797,9 @@ def is_zip_file(content):
 
 
 def unzip(path, target_dir):
+    if is_alpine():
+        # Running the native command can be an order of magnitude faster in Alpine on Travis-CI
+        return run('cd %s; unzip %s' % (target_dir, path))
     try:
         zip_ref = zipfile.ZipFile(path, 'r')
     except Exception as e:
@@ -781,9 +807,11 @@ def unzip(path, target_dir):
         raise e
     # Make sure to preserve file permissions in the zip file
     # https://www.burgundywall.com/post/preserving-file-perms-with-python-zipfile-module
-    for file_entry in zip_ref.infolist():
-        _unzip_file_entry(zip_ref, file_entry, target_dir)
-    zip_ref.close()
+    try:
+        for file_entry in zip_ref.infolist():
+            _unzip_file_entry(zip_ref, file_entry, target_dir)
+    finally:
+        zip_ref.close()
 
 
 def _unzip_file_entry(zip_ref, file_entry, target_dir):
@@ -796,6 +824,28 @@ def _unzip_file_entry(zip_ref, file_entry, target_dir):
     os.chmod(out_path, perm or 0o777)
 
 
+def untar(path, target_dir):
+    mode = 'r:gz' if path.endswith('gz') else 'r'
+    with tarfile.open(path, mode) as tar:
+        tar.extractall(path=target_dir)
+
+
+def zip_contains_jar_entries(content, jar_path_prefix=None, match_single_jar=True):
+    try:
+        with tempfile.NamedTemporaryFile() as tf:
+            tf.write(content)
+            tf.flush()
+            with zipfile.ZipFile(tf.name, 'r') as zf:
+                jar_entries = [e for e in zf.infolist() if e.filename.lower().endswith('.jar')]
+                if match_single_jar and len(jar_entries) == 1 and len(zf.infolist()) == 1:
+                    return True
+                matching_prefix = [e for e in jar_entries if
+                    not jar_path_prefix or e.filename.lower().startswith(jar_path_prefix)]
+                return len(matching_prefix) > 0
+    except Exception:
+        return False
+
+
 def is_jar_archive(content):
     """ Determine whether `content` contains valid zip bytes representing a JAR archive
         that contains at least one *.class file and a META-INF/MANIFEST.MF file. """
@@ -952,6 +1002,10 @@ def clone(item):
     return json.loads(json.dumps(item))
 
 
+def clone_safe(item):
+    return clone(json_safe(item))
+
+
 def remove_non_ascii(text):
     # text = unicode(text, "utf-8")
     text = text.decode('utf-8', CODEC_HANDLER_UNDERSCORE)
